/*
 * Global
 * ======
 *
 *  Demonstration of a non-atomicity of integer incrementation.
 *
 *  *tags*: test, C99
 *
 * Description
 * -----------
 *
 *  This program is a simple demonstration of a non-atomicity of integer incrementation,
 *  showing that such operation should be protected with mutex to be immune from
 *  concurrent access.
 *
 *  In this example, locking is disabled and therefore assertion violation is detected.
 *
 * Verification
 * ------------
 *
 *  - introducing a bug:
 *
 *         $ divine cc global-bug.c
 *         $ divine verify global-bug.bc
 *
 * Execution
 * ---------
 *
 *       $ clang -lpthread -o global global.c
 *       $ ./global
 */

#include <pthread.h>
#include <assert.h>
#include <stdlib.h>

volatile int i = 33;

pthread_mutex_t mutex;

void* thread( void *x ) {
    (void)x;

    i ++;

    return NULL;
}

int main() {
    pthread_t tid;

    pthread_mutex_init( &mutex, NULL );
    pthread_create( &tid, NULL, thread, NULL );

    pthread_mutex_lock( &mutex );
    i ++;
    pthread_mutex_unlock( &mutex );

    pthread_join( tid, NULL );
    assert( i == 35 );
    return 0;
}
