#include <thread>
#include <cassert>
#include <condition_variable>
#include <mutex>
#include <atomic>

// divine verify -std=c++14 -o nofail:malloc FILE

std::mutex lock;
std::condition_variable cond;
using Guard = std::unique_lock< std::mutex >;

// only accessed in critical section guarded by mutex, so there is no need to
// make these variables atomic or volatile
int x;
bool x_set = false;

void thread() {
    Guard guard( lock );
    cond.wait( guard, [&] { return x_set; } ); // wait until x_set is true
    assert( x == 42 );
}

int main() {
    std::thread t( thread );
    for ( int i = 0; i <= 42; i++ )
        x = i;
    x_set = true;
    cond.notify_all();
    t.join();

